@@ -1,21 +1,24 @@
-#!/usr/bin/env texlua
 -------------------------------------------------------------------------------
 --         FILE:  luaotfload-configuration.lua
---  DESCRIPTION:  config file reader
--- REQUIREMENTS:  Luaotfload 2.9 or above
+--  DESCRIPTION:  part of luaotfload / luaotfload-tool / config file reader
 --       AUTHOR:  Philipp Gesang, <phg@phi-gamma.net>
 --       AUTHOR:  Dohyun Kim <nomosnomos@gmail.com>
 -------------------------------------------------------------------------------
---
 
-if not modules then modules = { } end modules ["luaotfload-configuration"] = {
-  version   = "2.91",
-  comment   = "part of Luaotfload",
-  author    = "Philipp Gesang, Dohyun Kim",
-  copyright = "Luaotfload Development Team",
-  license   = "GNU GPL v2.0"
+local ProvidesLuaModule = { 
+    name          = "luaotfload-configuration",
+    version       = "2.94",       --TAGVERSION
+    date          = "2018-12-19", --TAGDATE
+    description   = "luaotfload submodule / config file reader",
+    license       = "GPL v2.0"
 }
 
+if luatexbase and luatexbase.provides_module then
+  luatexbase.provides_module (ProvidesLuaModule)
+end  
+
+------------------------------
+
 local status_file             = "luaotfload-status"
 local luaotfloadstatus        = require (status_file)
 
@@ -778,7 +781,7 @@ local resolve_config_path = function ()
     local t, p = unpack (config_paths[i])
     local fullname
     if t == kpse_t then
-      fullname = kpse.lookup (p)
+      fullname = kpse.find_file (p)
       logreport ("both", 6, "conf", "kpse lookup: %s -> %s.", p, fullname)
     elseif t == path_t then
       local expanded = tilde_expand (p)
